#!/usr/bin/python
from math import pi, sqrt, floor, cos, sin
from util import prefix_average, linspace, regular_plot

def N(t):
    'Implements N(t) from equation (1.1)'

    r = sqrt(t) / (2 * pi)
    n = 4 * int(r) + 1

    for x in xrange(1, int(r)+1, 1):
        y = sqrt(r*r - x*x)
        n += 4 * int(y)

    return n

def D(t):
    'Implements D(t) from equation (1.2)'
    return N(t) - t/(4 * pi)

def norm(x,y):
    return sqrt(x*x + y*y)

def g(t, n_bound = 10):
    acc = 0.0
    n_sq_norm = n_bound * n_bound

    for x in range(-n_bound, n_bound+1):
        for y in range(-n_bound, n_bound+1):
            if (x or y) and (x*x + y*y <= n_sq_norm):
                acc += norm(x,y)**(-2.5) * cos(norm(x,y) * t - pi/4)

    return acc * -sqrt(2*pi)/(pi*pi)

def A(R, step):
    'Returns an array of A(r) from equation (1.3) for r in [0, R]'
    x = linspace(int(R/step), 0, R)
    y = [D(t) for t in x]
    return (x, prefix_average(y))

def Atilde(R, step = 20):
    x  = linspace(int(R/step), 0, R)
    y0 = [D(4*pi*pi*r*r) for r in x]
    y  = [integral / r if r else 0 for r, integral in zip(x, prefix_average(y0))]
    return (x,y)

def Atilde2(R, step = 10):
    x0 = linspace(int(R/step), 0, 2*pi*R*R)
    y0 = [D(t) / sqrt(t) if t else 0 for t in x0]
    x = [sqrt(t/(2*pi)) for t in x0]
    y = [1/sqrt(8*pi*t) * ty if t else 0 for t, ty in zip(x, prefix_average(y0))]
    return (x,y)

def slowtilde1(R, step = 10):
    if not (int(R) % 10):
        print R
    x0 = linspace(int(R/step), 0, R)
    integrand = [D((2*pi*r)*(2*pi*r)) for r in x0]
    return sum(integrand)/len(integrand)

def slowtilde2(R, step = 10):
    print R

    bound = 2*pi*R*R
    x0 = linspace(int(bound/step), 0, bound)
    integrand = [D(t) / sqrt(t) if t else 0 for t in x0]
    return 0.5*R*sum(integrand)/max(len(integrand), 1)

def fig1(R, step = 10, out = 'figs/circ-fig-1.pdf', bounds = None):
    x = linspace(int(R/step), 0, R)
    y = map(D, x)

    regular_plot([(x,y)],
                 [],
                 ['$t$'],
                 ['$D(t)$'],
                 [''],
                 out,
                 bounds)

def fig2(R, step = 10, out = 'figs/circ-fig-2.pdf', bounds = None):
    x = linspace(int(R/step), 1, R)
    y = map(lambda t : D(t) * (t**-0.25), x)

    regular_plot([(x,y)],
                 [],
                 ['$t$'],
                 ['$t^{-1/4} D(t)$'],
                 [''],
                 out,
                 bounds)

def fig3(R, step = 10, out = 'figs/circ-fig-3.pdf', bounds = None):
    x, y = A(R, step)

    regular_plot([(x,y)],
                 [],
                 ['$t$'],
                 ['$A(t)$'],
                 [''],
                 out,
                 bounds)

def fig4(R, step = 10, out = 'figs/circ-fig-4.pdf', bounds = None):
    x, y0 = A(R, step)
    y = [ar * sqrt(sqrt(t)) for t, ar in zip(x, y0)]

    regular_plot([(x,y)],
                 [],
                 ['$t$'],
                 ['$t^{1/4}A(t)$'],
                 [''],
                 out,
                 bounds)

def fig5(R, step = 10, out = 'figs/circ-fig-5.pdf', bounds = None):
    x = linspace(int(R/step), 1, R)
    y = map(lambda t: g(sqrt(t)), x)

    regular_plot([(x,y)],
                 [],
                 ['$t$'],
                 ['$g(\sqrt{t})$'],
                 [''],
                 out,
                 bounds)

def fig6(R, step = 1, out = 'figs/circ-fig-6.pdf', bounds = None):
    x, y0 = A(R, step)
    y = [t**(0.25) * ay - g(sqrt(t)) for t, ay in zip(x,y0)]

    regular_plot([(x,y)],
                 [],
                 ['$t$'],
                 ['$t^{1/4}A(t) - g(\sqrt{t})$'],
                 [''],
                 out,
                 bounds)

def fig7(R, step = 1, out = 'figs/circ-fig-7.pdf', bounds = None):
    x, y0 = A(R, step)
    y = [sqrt(t) * (t**(0.25) * ay - g(sqrt(t))) for t, ay in zip(x,y0)]

    regular_plot([(x,y)],
                 [],
                 ['$t$'],
                 ['$\sqrt{t}(t^{1/4}A(t) - g(\sqrt{t}))$'],
                 [''],
                 out,
                 bounds)

# def g_without_sqrt(R, step = 10, out = 'figs/g(t).pdf', bounds = None):
#     x = linspace(int(R/step), 1, R)
#     y = [g(t, n_bound = 100) for t in x]

#     regular_plot([(x,y)],
#                  [],
#                  ['$t$'],
#                  ['$g(t)$'],
#                  [''],
#                  out,
#                  bounds)

def fig8(R, step = 10, out = 'figs/circ-fig-8.pdf', bounds = None):
    # x = linspace(int(R/step), 1, R)
    # y = [slowtilde2(r, step) for r in x]

    x, y = Atilde(R, step)
    regular_plot([(x,y)],
                 [],
                 ['$t$'],
                 [r'$\tilde A(t)$'],
                 [''],
                 out,
                 bounds)

# def fig62(R, step = 10, out = 'circ-fig-62.pdf', bounds = None):
#     x, y = Atilde2(R, step)

#     regular_plot([(x,y)],
#                  [],
#                  ['$t$'],
#                  [r'$\tilde A(t)$'],
#                  [''],
#                  out,
#                  bounds)

def fig9(R, step = 100, out = 'figs/circ-fig-9.pdf', bounds = None):
    bound = 2*pi*R*R
    x0, y0 = A(bound, step)
    x = [sqrt(t/(2 * pi)) for t in x0]
    y = [1/(2 * sqrt(2 * pi)) * t for t in y0]

    regular_plot([(x,y)],
                 [],
                 ['$t$'],
                 ['$A(2 \pi t^2)/(2 \sqrt{2 \pi})$'],
                 [''],
                 out,
                 bounds)

if __name__ == '__main__':
    fig6(10000, step = 0.1, bounds=((0, -0.5), (10000, 0.5)))
    fig7(10000, step = 0.1, bounds=((0, -0.5), (10000, 0.5)))
    # fig62(1000, step = 1)

    # # fig4(200000, step = 10, bounds = ((190000,-2),(200000,2)))
    # # fig5(200000, step = 100, bounds = ((190000,-2),(200000,2)))
    # g_without_sqrt(10000, step = 10, bounds = ((0,-2),(10000,2)))
