#!/usr/bin/python
from circ_comp import *
from util import *
from math import sqrt, cos, pi
import os
import cPickle

def single_batched_g(t, n_vals, n_grid):
    sep = n_vals[1] - n_vals[0]
    bound = n_grid[2]

    buckets = [0 for _ in n_vals]
    def insert_bucket(x, y):
        norm = sqrt(x*x + y*y)
        buckets[int((norm - n_grid[1])/sep)] += norm**(-5/2) * cos(norm*t - pi/4)

    for x in range(-bound, bound+1):
        for y in range(-bound, bound+1):
            if (x or y) and (x*x + y*y <= bound*bound):
                insert_bucket(x,y)

    buckets = [v * -sqrt(2*pi)/(pi*pi) for v in buckets]

    return {n : sum(buckets[:(i+1)]) for i, n in enumerate(n_vals)}

def batched_g(t_grid, n_grid):
    t_vals = linspace(*t_grid)
    n_vals = linspace(*n_grid)

    result = {n : [] for n in n_vals}

    for t in t_vals:
        print t
        for n, y in single_batched_g(sqrt(t), n_vals, n_grid).iteritems():
            result[n].append(y)

    return result

def max_norm(l):
    return max([sqrt(x*x+y*y) for x,y in l])

def g_convergence_compute(x_grid = (1000, 1, 10000),
                          n_grid = (10, 1, 500),
                          path = 'data/g_sqrt_convergence'):
    # x_vals = linspace(*x_grid)
    # n_vals = [int(n) for n in linspace(*n_grid)]

    # y_vals = {}

    # for n in n_vals:
    #     print 'n = %d' % n
    #     y_vals[n] = [g(x, n_bound = n, lines = False) for x in x_vals]

    y_vals = batched_g(x_grid, n_grid)

    data = {'x_grid' : x_grid,
            'y_vals' : y_vals,
            'n_grid' : n_grid}

    with open(path, 'w') as f:
        cPickle.dump(data, f)

def batched_A(x_grid = (1000, 1, 10000),
              strides = [2**i for i in range(11)],
              path = 'data/A_convergence'):
    largest = strides[-1]
    x = linspace(*x_grid)

    x_comp = linspace(x_grid[0]*largest, x_grid[1], x_grid[2])
    print x_grid[0] * largest
    y = [sqrt(sqrt(t)) * D(t) for t in x_comp]

    print 'done computing'

    sep = x_comp[1] - x_comp[0]
    y_vals = {1/float(sep*i) : prefix_average(y[::i])[::largest/i] for i in strides}

    with open(path, 'w') as f:
        data = {'y_vals' : y_vals,
                'x_grid' : x_grid,
                'strides' : strides}
        cPickle.dump(data, f)

def simultaneous_plots(data = 'data/g_sqrt_convergence',
                       out = 'figs/g_convergence_zoom.pdf'):
    with open(data, 'r') as f:
        d = cPickle.load(f)
        x_grid = d['x_grid']
        y_vals = d['y_vals']

    plots = []
    labels = []
    x_vals = linspace(*x_grid)

    print x_grid

    for n, y in sorted(y_vals.items(), key=lambda x: x[0])[::2]:
        print n
        plots.append((x_vals, y))
        labels.append(['$n = %d$' % n])

    # bound1 = ((5000,-2),(10000,2))
    bound1 = ((9868.5, 0.3305), (9869.5, 0.3345))
    bound2 = ((8750,-1.5),(9250,-0.5))

    regular_plot(plots, labels,
                 ['$t$'],
                 ['$g(\sqrt{t})$'],
                 ['Convergence of', '$g(\sqrt{t})$'],
                 out,
                 bounds=bound1)

def norm_difference(metric, plot_opts, out, data = 'data/g_convergence'):
    with open(data, 'r') as f:
        d = cPickle.load(f)
        x_grid = d['x_grid']
        y_vals = d['y_vals']

    curves = y_vals.items()
    curves.sort(key = lambda x : x[0])
    n_best, best = curves[-1]

    pairs = [(n, metric(best, y)) for n, y in curves[:-1]]
    x, y = zip(*pairs)

    loglog_plot([(x,y)], [],
                plot_opts['xlabel'],
                plot_opts['ylabel'](n_best),
                plot_opts['title'],
                out)

def sup_metric(f, g):
    return max([abs(x-y) for x,y in zip(f,g)])

def l1_metric(f, g):
    return sum([abs(x-y) for x,y in zip(f,g)])

def find_max_diff(f, g):
    x, y = None, 0
    for i, (a, b) in enumerate(zip(f,g)):
        if abs(a-b) >= y:
            x = i
            y = abs(a-b)

    return x,y

if __name__ == '__main__':
    # g_convergence_compute()
    simultaneous_plots()

    # batched_a()
    # simultaneous_plots()

    # with open('data/g_convergence_3', 'r') as f:
    #     x_grid = cPickle.load(f)['x_grid']

    # title = ['Convergence of', '$g$', 'in the sup norm on',
    #          '$[%d,%d]$' % (x_grid[1], x_grid[2])]

    # l1title = ['Convergence of', '$g$', 'in the', '$\ell_1$', 'norm on',
    #            '$[%d,%d]$' % (x_grid[1], x_grid[2])]

    # plot_opts = {'xlabel' : ['$n$'],
    #              'ylabel' : lambda n_best : ['$||g_{%d} - g_n||_\infty$' % n_best],
    #              'title' : title}

    # l1plot_opts = {'xlabel' : ['$n$'],
    #                'ylabel' : lambda n_best : ['$||A_{%d} - A_n||_1$' % n_best],
    #                'title' : l1title}

    # norm_difference(sup_metric, plot_opts, 'figs/g-sup-convergence.pdf',
    #                 'data/g_convergence_3')
    # norm_difference(l1_metric, l1plot_opts, 'figs/g-l1-convergence.pdf',
    #                 'data/g_convergence_3')

    # with open('data/A_convergence', 'r') as f:
    #     a_x_grid = cPickle.load(f)['x_grid']

    # a_title = ['Convergence of', '$A$', 'in the sup norm on',
    #            '$[%d,%d]$' % (a_x_grid[1], a_x_grid[2])]

    # a_l1title = ['Convergence of', '$A$', 'in the', '$\ell_1$', 'norm on',
    #              '$[%d,%d]$' % (a_x_grid[1], a_x_grid[2])]

    # a_plot_opts = {'xlabel' : ['$n$', '(sample points per unit)'],
    #                'ylabel' : lambda n_best : ['$||A_{%d} - A_n||_\infty$' % n_best],
    #                'title' : a_title}

    # a_l1plot_opts = {'xlabel' : ['$n$', '(sample points per unit)'],
    #                  'ylabel' : lambda n_best : ['$||A_{%d} - A_n||_1$' % n_best],
    #                  'title' : a_l1title}

    # norm_difference(sup_metric, a_plot_opts,
    #                 'figs/a-sup-convergence.pdf', 'data/A_convergence')
    # norm_difference(l1_metric, a_l1plot_opts,
    #                 'figs/a-l1-convergence.pdf', 'data/A_convergence')
