#!/usr/bin/python
from math import pi, sqrt, cos
from util import linspace, prefix_average, regular_plot
import cPickle

def N(t, a1, a2):
    v_axis = a2 * sqrt(t) / (2 * pi)
    h_axis = a1 * sqrt(t) / (2 * pi)

    iv_axis = int(v_axis)
    ih_axis = int(h_axis)

    n = 1 + 2*iv_axis + 2*ih_axis

    for x in xrange(1, ih_axis+1):
        n += 4 * int(sqrt( v_axis**2 - (a2*x/a1)**2 ))

    return n

def D(t, a1, a2):
    return N(t, a1, a2) - a1*a2*t/(4*pi)

def A(R, a1, a2, step):
    x = linspace(int(R/step), 0, R)
    y = [D(t,a1,a2) for t in x]
    return x, prefix_average(y)

def g(t, a1, a2, n_bound):
    acc = 0.
    n_norm = sqrt(n_bound * n_bound)

    for x in xrange(-n_bound, n_bound+1):
        for y in xrange(-n_bound, n_bound+1):
            norm = sqrt(x*x + y*y)
            if (x or y) and (norm <= n_norm):
                el_norm = sqrt((a1*x)**2. + (a2*y)**2.)
                acc += el_norm**(-2.5) * cos(el_norm * t - pi/4)

    return -a1*a2*sqrt(2)/(pi**1.5) * acc

def compute(R, a1, a2, step, g_bound):
    x, Ay = A(R, a1, a2, step)
    gy    = [g(sqrt(t), a1, a2, g_bound) for t in x]

    Aout = [ax**0.25 * ay for ax, ay in zip(x,Ay)]
    out = (x, Aout, gy)

    name = 'data/ellipse/R=%f-a1=%f-a2=%f-step=%f-g_bound=%d'\
        % (R, a1, a2, step, g_bound)
    with open(name, 'w') as f:
        cPickle.dump(out, f)

def plot_one(data, Aout, gout, bounds = None):
    with open(data, 'r') as f:
        x, Ay, gy = cPickle.load(f)

    args = dict([s.split('=') for s in data.split('-')])
    a1 = float(args['a1'])
    a2 = float(args['a2'])

    atitle = ['$t^{1/4}A(t)$', 'for', '$a_1 = %f$' % a1, 'and', '$a_2 = %f$' % a2]
    gtitle = ['$g(\sqrt{t})$', 'for', '$a_1 = %f$' % a1, 'and', '$a_2 = %f$' % a2]

    atitle = gtitle = ['']

    regular_plot([(x, Ay)],
                 [],
                 ['$t$'],
                 ['$t^{1/4}A(t)$'],
                 atitle,
                 Aout,
                 bounds)

    regular_plot([(x, gy)],
                 [],
                 ['$t$'],
                 ['$g(\sqrt{t})$'],
                 gtitle,
                 gout,
                 bounds)

def plot_diff(data, dout, bounds = None):
    with open(data, 'r') as f:
        x, Ay, gy = cPickle.load(f)

    args = dict([s.split('=') for s in data.split('-')])
    a1 = float(args['a1'])
    a2 = float(args['a2'])

    diff = [a - b for a, b in zip(Ay, gy)]

    title = ['Difference for', '$a_1 = %f$' % a1, 'and', '$a_2 = %f$' % a2]
    title = ['']

    regular_plot([(x, diff)],
                 [],
                 ['$t$'],
                 ['$t^{1/4}A(t) - g(\sqrt{t})$'],
                 title,
                 dout,
                 bounds)


if __name__ == '__main__':
    import os

    R = 10000.
    step = 0.1
    arg_l = [(R, a1, 1./a1, step, 10) for a1 in [2]]

    for args in arg_l:
        print args
        compute(*args)

    names = ['data/ellipse/R=%f-a1=%f-a2=%f-step=%f-g_bound=%d'\
        % arg for arg in arg_l]

    for i, n in list(enumerate(names)):
        plot_one(n, 'out/a%d.pdf' % i, 'out/g%d.pdf' % i, bounds = ((0,-4),(10000,4)))
        plot_diff(n, 'out/d%d.pdf' % i, bounds = ((0,-2),(10000,2)))
