#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#define PI 3.141592654
#define LEN(r,step) (1+(int)r/step)

static int counter = 0;

typedef struct {
    double x;
    double y;
} point;

inline double norm(double x, double y) {
    return sqrt(x*x + y*y);
}

double g(double t, int n_bound) {
    double acc = 0;
    int n_sq_norm = n_bound * n_bound;
    int x, y;

    for (x = -n_bound; x <= n_bound; ++x)
        for (y = -n_bound; y <= n_bound; ++y)
            if ((x || y) && (x*x + y*x <= n_sq_norm))
                acc += pow(norm(x,y), -2.5) * cos(norm(x,y)*t - PI/4);

    return acc * -sqrt(2*PI)/(PI*PI);
}

double D(double t) {
    /* Computes N(t) - t/(4*pi) */
    int x, y;

    double r = sqrt(t) / (2*PI);
    int r0 = (int) r;
    long long n = 4*r0 + 1;


    for (x = 1; x <= r0; ++x) {
        y = (int) sqrt(r*r - x*x);
        n += 4 * y;
    }

    return n - t/(4*PI);
}

point* g_sqrt_array(double R, double step) {
    int i;
    double t;
    point* vals = calloc(LEN(R,step), sizeof(point));

    for (i = 0; i < LEN(R,step); ++i) {
        if (!(counter++ % 100))
            printf("%i\n", i);

        t = i*step;
        vals[i].x = t;
        vals[i].y = g(sqrt(t), 100);
    }

    return vals;
}

point* A(double R, double step) {
    int i;
    double t;
    double accum = 0;
    point* vals = calloc(LEN(R,step), sizeof(point));

    for (i = 0; i < LEN(R,step); ++i) {
        t = i*step;
        accum += D(t);
        vals[i].x = t;
        vals[i].y = sqrt(sqrt(t)) * accum/(i+1);
    }

    return vals;
}

void writePoints(point* A, point* g, double R, double step, char* path) {
    int i;
    FILE* fh = fopen(path, "w");
    char line[80];

    fprintf(fh, "t t^{1/4}A(t) g(sqrt{t})\n");

    for (i = 0; i < LEN(R,step); ++i)
        fprintf(fh, "%f %f %f\n",
                (float) A[i].x,
                (float) A[i].y,
                (float) g[i].y);

    fclose(fh);
}

int main(int argc, char** argv) {
    float _R, _step;

    if (argc < 4) {
        printf("Usage: R step out\n");
        return -1;
    }

    sscanf(argv[1], "%f", &_R);
    sscanf(argv[2], "%f", &_step);

    double R = (double) _R;
    double step = (double) _step;

    point* A_array = A(R, step);
    point* g_array = g_sqrt_array(R, step);

    writePoints(A_array, g_array, R, step, argv[3]);
    return 0;
}
